/*

    File: file_doc.c

    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 59
    Temple Place - Suite 330, Boston MA 02111-1307, USA.

 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#include "types.h"
#include "photorec.h"

static const char* header_check_doc(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery);
const t_file_hint file_hint_doc= {
  .extension="doc",
  .description="Microsoft Office Document",
  .min_header_distance=0,
  .min_filesize=0,
  .max_filesize=PHOTOREC_MAX_FILE_SIZE,
  .recover=1,
  .header_check=&header_check_doc,
  .data_check=NULL,
  .file_check=NULL
};

static const unsigned char * find_in_mem(const unsigned char *haystack, const unsigned char *needle, const unsigned int haystack_size);
static const unsigned char * find_in_mem(const unsigned char *haystack, const unsigned char *needle, const unsigned int haystack_size)
{
  unsigned int i;
  unsigned int needle_size=strlen(needle);
  for(i=0;i<haystack_size-needle_size;i++)
    if(memcmp(&haystack[i],needle,needle_size)==0)
      return &haystack[i];
  return NULL;
}

static const char* header_check_doc(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery)
{
  const unsigned char doc_header[]= { 0xd0, 0xcf, 0x11, 0xe0, 0xa1, 0xb1, 0x1a, 0xe1};
  if(memcmp(buffer,doc_header,sizeof(doc_header))==0)
  {
    /* TODO try to understand OLE format */
    if(find_in_mem(buffer,"WordDocument",buffer_size))
    {
      return "doc";
    }
    else if(find_in_mem(buffer,"StarDraw",buffer_size))
    {
      return "sda";
    }
    else if(find_in_mem(buffer,"StarCalc",buffer_size))
    {
      return "sdc";
    }
    else if(find_in_mem(buffer,"StarImpress",buffer_size))
    {
      return "sdd";
    }
    else if(find_in_mem(buffer,"Worksheet",buffer_size) || find_in_mem(buffer,"Book",buffer_size) || find_in_mem(buffer,"Workbook",buffer_size) || find_in_mem(buffer,"Calc",buffer_size))
    {
      return "xls";
    }
    else if(find_in_mem(buffer,"Power",buffer_size))
    {
      return "ppt";
    }
    else if(find_in_mem(buffer,"Access",buffer_size) || find_in_mem(buffer,"AccessObjSiteData",buffer_size))
    {
      return "mbd";
    }
    else if(find_in_mem(buffer,"Visio",buffer_size))
    {
      return "vis";
    }
    else if(find_in_mem(buffer,"Sfx",buffer_size))
    {
      return "sdw";
    }
    return file_hint_doc.extension;
  }
  return NULL;
}


