/*

    File: file_mov.c

    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 59
    Temple Place - Suite 330, Boston MA 02111-1307, USA.

 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#include "types.h"
#include "photorec.h"

static const char* header_check_mov(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery);
const t_file_hint file_hint_mov= {
  .extension="mov",
  .description="MOV video",
  .min_header_distance=0,
  .min_filesize=0,
  .max_filesize=PHOTOREC_MAX_FILE_SIZE,
  .recover=1,
  .header_check=&header_check_mov,
  .data_check=NULL,
  .file_check=NULL
};

static const unsigned char * find_in_mem(const unsigned char *haystack, const unsigned char *needle, const unsigned int haystack_size);
static const unsigned char * find_in_mem(const unsigned char *haystack, const unsigned char *needle, const unsigned int haystack_size)
{
  unsigned int i;
  unsigned int needle_size=strlen(needle);
  for(i=0;i<haystack_size-needle_size;i++)
    if(memcmp(&haystack[i],needle,needle_size)==0)
      return &haystack[i];
  return NULL;
}
static const char* header_check_mov(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery)
{
  unsigned int atom_size=(buffer[0]<<24)+(buffer[1]<<16)+(buffer[2]<<8)+buffer[3];
  /* check for commun atom type */
  if(atom_size>=8 &&
   ((atom_size < 256 && buffer[4]=='p' && buffer[5]=='n' && buffer[6]=='o' && buffer[7]=='t') ||
   (atom_size < 256 && buffer[4]=='w' && buffer[5]=='i' && buffer[6]=='d' && buffer[7]=='e') ||
   (atom_size < 256*256*256 && buffer[4]=='m' && buffer[5]=='o' && buffer[6]=='o' && buffer[7]=='v')
   ))
    return file_hint_mov.extension;
  if(atom_size>=8 && buffer[4]=='f' && buffer[5]=='t' && buffer[6]=='y' && buffer[7]=='p')
  {
    if(find_in_mem(&buffer[8], "isom",atom_size-8)||
	find_in_mem(&buffer[8], "mp41",atom_size-8)||
	find_in_mem(&buffer[8], "mp42",atom_size-8)||
	find_in_mem(&buffer[8], "3gp",atom_size-8)||
	find_in_mem(&buffer[8], "mmp4",atom_size-8)||
	find_in_mem(&buffer[8], "M4A",atom_size-8)||
	find_in_mem(&buffer[8], "M4P",atom_size-8)||
	find_in_mem(&buffer[8], "M4B",atom_size-8))
      return "mp4";
    if(find_in_mem(&buffer[8], "jp2",atom_size-8))
      return "jpg";
    if(find_in_mem(&buffer[8], "qt",atom_size-8))
      return "mov";
  }
  return NULL;
}


